# Copyright (c)  2020  Xiaomi Corporation (authors: Fangjun Kuang)

import _kaldilm


def arpa2fst(input_arpa: str,
             output_fst: str,
             bos_symbol: str = '<s>',
             disambig_symbol: str = '',
             eos_symbol: str = '</s>',
             ilabel_sort: bool = True,
             keep_symbols: bool = False,
             max_arpa_warnings: int = 30,
             read_symbol_table: str = '',
             write_symbol_table: str = '') -> str:
    '''Convert an ARPA file to an FST.

    This function is a wrapper of kaldi's arpa2fst and
    arguments names meanings are not changed.

    Args:
      The input arpa file.
    output_fst:
      The output fst file. Note that it is a binary file.
      This function will return a text format of it.
    bos_symbol:
      Beginning of sentence symbol.
    disambig_symbol:
      Disambiguator. If provided (e.g., #0), used on input side of backoff
      links, and <s> and </s> are replaced with epsilons.
    eos_symbol:
      End of sentence symbol.
    ilabel_sort:
      Ilabel-sort the output FST.
    keep_symbols:
      Store symbol table with FST. Symbols always saved to FST if symbol
      tables are neither read or written (otherwise symbols would be lost
      entirely).
    max_arpa_warnings:
      Maximum warnings to report on ARPA parsing, 0 to disable, -1 to
      show all.
    read_symbol_table:
      use existing symbol table.
    write_symbol_table:
      Write generated symbol table to a file.

    Returns:
      Return a text format of the resulting FST with integer labels.
    '''
    s = _kaldilm.arpa2fst(input_arpa=input_arpa,
                          output_fst=output_fst,
                          bos_symbol=bos_symbol,
                          disambig_symbol=disambig_symbol,
                          eos_symbol=eos_symbol,
                          ilabel_sort=ilabel_sort,
                          keep_symbols=keep_symbols,
                          max_arpa_warnings=max_arpa_warnings,
                          read_symbol_table=read_symbol_table,
                          write_symbol_table=write_symbol_table)
    return s



#!/usr/bin/env python3
#
# Copyright (c)  2020  Xiaomi Corporation (author: Fangjun Kuang)

import glob
import os
import re
import setuptools
import shutil
import subprocess

from setuptools.command.build_ext import build_ext

cur_dir = os.path.dirname(os.path.abspath(__file__))


def cmake_extension(name, *args, **kwargs) -> setuptools.Extension:
    kwargs['language'] = 'c++'
    sources = []
    return setuptools.Extension(name, sources, *args, **kwargs)


class BuildExtension(build_ext):
    def build_extension(self, ext: setuptools.extension.Extension):
        build_dir = f'{cur_dir}/build'
        os.makedirs(build_dir, exist_ok=True)

        os.makedirs(self.build_lib, exist_ok=True)

        os.system(f'cd {build_dir}; cmake ..; make -j _kaldilm')
        lib_so = glob.glob(f'{build_dir}/lib/*.so')
        for so in lib_so:
            shutil.copy(f'{so}', f'{self.build_lib}/')


def read_long_description():
    with open('README.md', encoding='utf8') as f:
        readme = f.read()
    return readme


def get_package_version():
    with open('CMakeLists.txt') as f:
        content = f.read()

    latest_version = re.search(r'set\(kaldilm_VERSION (.*)\)',
                               content).group(1)
    latest_version = latest_version.strip('"')
    return latest_version


package_name = 'kaldilm'

setuptools.setup(
    name=package_name,
    version=get_package_version(),
    author='Fangjun Kuang',
    author_email='csukuangfj@gmail.com',
    data_files=[('', ['LICENSE', 'README.md'])],
    package_dir={
        package_name: 'kaldilm/python/kaldilm',
    },
    packages=[package_name],
    url='https://github.com/csukuangfj/kaldilm',
    long_description=read_long_description(),
    long_description_content_type='text/markdown',
    ext_modules=[cmake_extension(package_name)],
    cmdclass={'build_ext': BuildExtension},
    zip_safe=False,
    classifiers=[
        'Programming Language :: C++',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
    ],
    license='Apache licensed, as found in the LICENSE file',
)
